@@ -8,12 +8,18 @@
 # text.set_position()
 # quiver.set_offsets(), quiver.set_UVC()
 # FancyArrow.set_xy()
-
+import os.path
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib import animation
+from numpy.lib.arraysetops import isin
 from spatialmath import base
+from collections.abc import Iterable, Generator, Iterator
+import time
 
+# global variable holds reference to FuncAnimation object, this is essential
+# for animatiion to work
+_ani = None
 
 class Animate:
     """
@@ -57,6 +63,7 @@ class Animate:
         Will setup to plot into an existing or a new Axes3D instance.
 
         """
+        self.trajectory = None
         self.displaylist = []
 
         if axes is None:
@@ -104,6 +111,16 @@ class Animate:
         :seealso: :func:`run`
 
         """
+        if not isinstance(end, (np.ndarray, np.generic) ) and isinstance(end, Iterable):
+            try:
+                if len(end) == 1:
+                    end = end[0]
+                elif len(end) >= 2:
+                    self.trajectory = end
+            except TypeError:
+                # a generator has no len()
+                self.trajectory = end
+
         # stash the final value
         if base.isrot(end):
             self.end = base.r2t(end)
@@ -119,9 +136,9 @@ class Animate:
                 self.start = start
 
         # draw axes at the origin
-        base.trplot(self.start, axes=self, block=None, **kwargs)
+        base.trplot(self.start, axes=self, **kwargs)
 
-    def run(self, movie=None, axes=None, repeat=False, interval=50, nframes=100, pause=0, **kwargs):
+    def run(self, movie=None, axes=None, repeat=False, interval=50, nframes=100, wait=False, **kwargs):
         """
         Run the animation
 
@@ -129,12 +146,14 @@ class Animate:
         :type axes: Axes3D reference
         :param repeat: animate in endless loop [default False]
         :type repeat: bool
-        :param nframes: number of steps in the animation [defaault 100]
+        :param nframes: number of steps in the animation [default 100]
         :type nframes: int
         :param interval: number of milliseconds between frames [default 50]
         :type interval: int
         :param movie: name of file to write MP4 movie into
         :type movie: str
+        :param wait: wait until animation is complete, default False
+        :type wait: bool
 
         Animates a 3D coordinate frame moving from the world frame to a frame
         represented by the SO(3) or SE(3) matrix to the current axes.
@@ -146,27 +165,64 @@ class Animate:
             - invokes the draw() method of every object in the display list
         """
 
-        def update(frame, a):
-            T = base.trinterp(start=self.start, end=self.end, s=frame / nframes)
-            a._draw(T)
-            if frame == nframes - 1:
-                a.done = True
-            return a.artists()
+        def update(frame, animation):
+
+            # frame is the result of calling next() on a iterator or generator
+            # seemingly the animation framework isn't checking StopException
+            # so there is no way to know when this is no longer called.
+            # we implement a rather hacky heartbeat style timeout
+
+            if isinstance(frame, float):
+                # passed a single transform, interpolate it
+                T = base.trinterp(start=self.start, end=self.end, s=frame)
+            else:
+                # assume it is an SO(3) or SE(3)
+                T = frame
+                
+            # ensure result is SE(3)
+            if T.shape == (3,3):
+                T = base.r2t(T)
+
+            # update the scene
+            animation._draw(T)
+
+            self.count += 1  # say we're still running
+
+            return animation.artists()
+
+        global _ani
 
         # blit leaves a trail and first frame
         if movie is not None:
             repeat = False
 
-        self.done = False
-        ani = animation.FuncAnimation(fig=plt.gcf(), func=update, frames=range(0, nframes), fargs=(self,), blit=False, interval=interval, repeat=repeat)
-        if movie is None:
-            while repeat or not self.done:
-                plt.pause(0.1)
+        self.count = 1
+        if self.trajectory is not None:
+            if not isinstance(self.trajectory, Iterator):
+                # make it iterable, eg. if a list or tuple
+                self.trajectory = iter(self.trajectory)
+            frames = self.trajectory
         else:
+            frames = iter(np.linspace(0, 1, nframes))
+
+        _ani = animation.FuncAnimation(fig=plt.gcf(), func=update, frames=frames, fargs=(self,), blit=False, interval=interval, repeat=repeat)
+
+        if movie is not None:
             # Set up formatting for the movie files
-            print('creating movie', movie)
-            FFwriter = animation.FFMpegWriter(fps=10, extra_args=['-vcodec', 'libx264'])
-            ani.save(movie, writer=FFwriter)
+            if os.path.exists(movie):
+                print('overwriting movie', movie)
+            else:
+                print('creating movie', movie)
+            FFwriter = animation.FFMpegWriter(fps=1000 / interval, extra_args=['-vcodec', 'libx264'])
+            _ani.save(movie, writer=FFwriter)
+
+        if wait:
+            # wait for the animation to finish.  Dig into the timer for this
+            # animation and wait for its callback to be deregistered.
+            while True:
+                plt.pause(0.25)
+                if len(_ani.event_source.callbacks) == 0:
+                    break
 
     def __repr__(self):
         """
@@ -398,6 +454,7 @@ class Animate2:
         Will setup to plot into an existing or a new Axes3D instance.
 
         """
+        self.trajectory = None
         self.displaylist = []
 
         if axes is None:
@@ -439,6 +496,12 @@ class Animate2:
         :seealso: :func:`run`
 
         """
+        if not isinstance(end, (np.ndarray, np.generic) ) and isinstance(end, Iterable):
+            if len(end) == 1:
+                end = end[0]
+            elif len(end) >= 2:
+                self.trajectory = end
+
         # stash the final value
         if base.isrot2(end):
             self.end = base.r2t(end)
@@ -482,7 +545,11 @@ class Animate2:
         """
 
         def update(frame, a):
-            T = base.trinterp2(start=self.start, end=self.end, s=frame / nframes)
+            # if contains trajectory:
+            if self.trajectory is not None:
+                T = self.trajectory[frame]
+            else:
+                T = base.trinterp2(start=self.start, end=self.end, s=frame / nframes)
             a._draw(T)
             if frame == nframes - 1:
                 a.done = True
@@ -493,6 +560,8 @@ class Animate2:
             repeat = False
 
         self.done = False
+        if self.trajectory is not None:
+            nframes = len(self.trajectory)
         ani = animation.FuncAnimation(fig=plt.gcf(), func=update, frames=range(0, nframes), fargs=(self,), blit=False, interval=interval, repeat=repeat)
 
         if movie is None:
@@ -500,7 +569,10 @@ class Animate2:
                 plt.pause(1)
         else:
             # Set up formatting for the movie files
-            print('creating movie', movie)
+            if os.path.exists(movie):
+                print('overwriting movie', movie)
+            else:
+                print('creating movie', movie)
             FFwriter = animation.FFMpegWriter(fps=10, extra_args=['-vcodec', 'libx264'])
             ani.save(movie, writer=FFwriter)
 
@@ -666,98 +738,28 @@ class Animate2:
     def set_ylabel(self, *args, **kwargs):
         self.ax.set_ylabel(*args, **kwargs)
 
-def plotvol2(dim, ax=None):
-    """
-    Create 2D plot area
-
-    :param ax: axes of initializer, defaults to new subplot
-    :type ax: AxesSubplot, optional
-    :return: initialized axes
-    :rtype: AxesSubplot
-
-    Initialize axes with dimensions given by ``dim`` which can be:
-
-        * A (scalar), -A:A x -A:A
-        * [A,B], A:B x A:B
-        * [A,B,C,D], A:B x C:D
-
-    :seealso: :func:`plotvol3`, :func:`expand_dims`
-    """
-    dims = expand_dims(dim, 2)
-    if ax is None:
-        ax = plt.subplot()
-    ax.axis(dims)
-    return ax
-
-def plotvol3(dim, ax=None):
-    """
-    Create 3D plot volume
-
-    :param ax: axes of initializer, defaults to new subplot
-    :type ax: Axes3DSubplot, optional
-    :return: initialized axes
-    :rtype: Axes3DSubplot
-
-    Initialize axes with dimensions given by ``dim`` which can be:
-
-        * A (scalar), -A:A x -A:A x -A:A
-        * [A,B], A:B x A:B x A:B
-        * [A,B,C,D,E,F], A:B x C:D x E:F
-
-    :seealso: :func:`plotvol2`, :func:`expand_dims`
-    """
-    dims = expand_dims(dim, 3)
-    print(dims)
-    if ax is None:
-        ax = plt.subplot(projection='3d')
-    ax.set_xlim3d(dims[0], dims[1])
-    ax.set_ylim3d(dims[2], dims[3])
-    ax.set_zlim3d(dims[4], dims[5])
-    return ax
-
-def expand_dims(dim=None, nd=2):
-    """[summary]
 
 
-    :param dim: [description], defaults to None
-    :type dim: [type], optional
-    :param nd: [description], defaults to 2
-    :type nd: int, optional
-    :raises ValueError: bad arguments
-    :return: 2d or 3d dimensions vector
-    :rtype: ndarray(4) or ndarray(6)
+if __name__ == "__main__":
+    # from spatialmath import UnitQuaternion
+    # from spatialmath.base import tranimate, r2t
 
-    Compute bounding dimensions for plots from shorthand notation.
+    # J = np.array([[2, -1, 0], [-1, 4, 0], [0, 0, 3]])
+    # dt = 0.05
+    # def attitude():
+    #     attitude = UnitQuaternion()
+    #     w = 0.2 * np.r_[1, 2, 2].T
+    #     for t in np.arange(0, 3, dt):
+    #         wd =  -np.linalg.inv(J) @ (np.cross(w, J @ w))
+    #         w += wd * dt
+    #         attitude.increment(w * dt)
+    #         yield attitude.R
+    # plt.figure()
+    # plotvol3(2)
+    # tranimate(attitude())
 
-    If ``nd==2``, [xmin, xmax, ymin, ymax]:
-        * A -> [-A, A, -A, A]
-        * [A,B] -> [A, B, A, B]
-        * [A,B,C,D] -> [A, B, C, D]
+    # from spatialmath import base
 
-    If ``nd==3``, [xmin, xmax, ymin, ymax, zmin, zmax]:
-        * A -> [-A, A, -A, A, -A, A]
-        * [A,B] -> [A, B, A, B, A, B]
-        * [A,B,C,D,E,F] -> [A, B, C, D, E, F]
-    """
-    dim = base.getvector(dim)
-
-    if nd == 2:
-        if len(dim) == 1:
-            return np.r_[-dim, dim, -dim, dim]
-        elif len(dim) == 2:
-                return np.r_[-dim[0], dim[0], -dim[1], dim[1]]
-        elif len(dim) == 4:
-                return dim
-        else:
-            raise ValueError('bad dimension specified')
-    elif nd == 3:
-        if len(dim) == 1:
-                return np.r_[-dim, dim, -dim, dim, -dim, dim]
-        elif len(dim) == 3:
-                return np.r_[-dim[0], dim[0], -dim[1], dim[1], -dim[2], dim[2]]
-        elif len(dim) == 6:
-                return dim
-        else:
-            raise ValueError('bad dimension specified')
-    else:
-        raise ValueError('nd is 2 or 3')
+    # T = base.rpy2r(0.3, 0.4, 0.5)
+    # base.tranimate(T, wait=True)
+    pass
